@@ -20,14 +20,10 @@
 #include "elisa_settings.h"
 #include <KAuthorized>
 #include <KLocalizedString>
+#include <KStandardActions>
 
-#if KFConfigWidgets_FOUND
-#include <KStandardAction>
-#endif
-
-#if defined KFXmlGui_FOUND
+#if KFXmlGui_FOUND
 #include <KActionCollection>
-#include <KAboutApplicationDialog>
 #include <KHelpMenu>
 #include <KBugReport>
 #include <KShortcutsDialog>
@@ -57,7 +53,6 @@
 #include <QMimeDatabase>
 #include <QMimeType>
 
-
 class ElisaApplicationPrivate
 {
 public:
@@ -88,8 +83,6 @@ public:
 
     std::unique_ptr<ManageHeaderBar> mManageHeaderBar;
 
-    QQmlEngine *mEngine = nullptr;
-
     QFileSystemWatcher mConfigFileWatcher;
 
     KColorSchemeManager *mSchemes = nullptr;
@@ -102,8 +95,11 @@ ElisaApplication::ElisaApplication(QObject *parent) : QObject(parent), d(std::ma
     Elisa::ElisaConfiguration::instance(configurationFileName);
     Elisa::ElisaConfiguration::self()->load();
     Elisa::ElisaConfiguration::self()->save();
-
+#if KCOLORSCHEME_VERSION < QT_VERSION_CHECK(6, 6, 0)
     d->mSchemes = new KColorSchemeManager(this);
+#else
+    d->mSchemes = KColorSchemeManager::instance();
+#endif
 
     d->mConfigFileWatcher.addPath(Elisa::ElisaConfiguration::self()->config()->name());
 
@@ -120,6 +116,11 @@ ElisaApplication::ElisaApplication(QObject *parent) : QObject(parent), d(std::ma
 ElisaApplication::~ElisaApplication()
 = default;
 
+KColorSchemeManager *ElisaApplication::getSchemes()
+{
+    return d->mSchemes;
+}
+
 QAbstractItemModel *ElisaApplication::colorSchemesModel()
 {
     return d->mSchemes->model();
@@ -134,37 +135,37 @@ void ElisaApplication::setupActions(const QString &actionName)
 {
 #if KFXmlGui_FOUND
     if (actionName == QLatin1String("file_quit")) {
-        auto quitAction = KStandardAction::quit(QCoreApplication::instance(), &QCoreApplication::quit, &d->mCollection);
+        auto quitAction = KStandardActions::quit(QCoreApplication::instance(), &QCoreApplication::quit, &d->mCollection);
         d->mCollection.addAction(actionName, quitAction);
     }
 
-    if (actionName == QLatin1String("help_contents") && KAuthorized::authorizeAction(actionName)) {
-        auto handBookAction = KStandardAction::helpContents(this, &ElisaApplication::appHelpActivated, &d->mCollection);
-        d->mCollection.addAction(handBookAction->objectName(), handBookAction);
-    }
-
     if (actionName == QLatin1String("help_report_bug") && KAuthorized::authorizeAction(actionName) && !KAboutData::applicationData().bugAddress().isEmpty()) {
-        auto reportBugAction = KStandardAction::reportBug(this, &ElisaApplication::reportBug, &d->mCollection);
+        auto reportBugAction = KStandardActions::reportBug(this, &ElisaApplication::reportBug, &d->mCollection);
         d->mCollection.addAction(reportBugAction->objectName(), reportBugAction);
     }
 
     if (actionName == QLatin1String("help_about_app") && KAuthorized::authorizeAction(actionName)) {
-        auto aboutAppAction = KStandardAction::aboutApp(this, &ElisaApplication::aboutApplication, this);
+        auto aboutAppAction = KStandardActions::aboutApp(this, &ElisaApplication::openAboutAppPage, this);
         d->mCollection.addAction(aboutAppAction->objectName(), aboutAppAction);
     }
 
+    if (actionName == QLatin1String("help_about_kde") && KAuthorized::authorizeAction(actionName)) {
+        auto aboutKDEAction = KStandardActions::aboutKDE(this, &ElisaApplication::openAboutKDEPage, this);
+        d->mCollection.addAction(aboutKDEAction->objectName(), aboutKDEAction);
+    }
+
     if (actionName == QLatin1String("options_configure") && KAuthorized::authorizeAction(actionName)) {
-        auto preferencesAction = KStandardAction::preferences(this, &ElisaApplication::configureElisa, this);
+        auto preferencesAction = KStandardActions::preferences(this, &ElisaApplication::configureElisa, this);
         d->mCollection.addAction(preferencesAction->objectName(), preferencesAction);
     }
 
     if (actionName == QLatin1String("options_configure_keybinding") && KAuthorized::authorizeAction(actionName)) {
-        auto keyBindingsAction = KStandardAction::keyBindings(this, &ElisaApplication::configureShortcuts, this);
+        auto keyBindingsAction = KStandardActions::keyBindings(this, &ElisaApplication::configureShortcuts, this);
         d->mCollection.addAction(keyBindingsAction->objectName(), keyBindingsAction);
     }
 
     if (actionName == QLatin1String("go_back") && KAuthorized::authorizeAction(actionName)) {
-        auto goBackAction = KStandardAction::back(this, &ElisaApplication::goBack, this);
+        auto goBackAction = KStandardActions::back(this, &ElisaApplication::goBack, this);
         d->mCollection.addAction(goBackAction->objectName(), goBackAction);
     }
 
@@ -205,7 +206,7 @@ void ElisaApplication::setupActions(const QString &actionName)
     }
 
     if (actionName == QLatin1String("edit_find") && KAuthorized::authorizeAction(actionName)) {
-        auto findAction = KStandardAction::find(this, &ElisaApplication::find, this);
+        auto findAction = KStandardActions::find(this, &ElisaApplication::find, this);
         d->mCollection.addAction(findAction->objectName(), findAction);
     }
 
@@ -217,6 +218,8 @@ void ElisaApplication::setupActions(const QString &actionName)
     }
 
     d->mCollection.readSettings();
+#else
+    Q_UNUSED(actionName)
 #endif
 }
 
@@ -274,23 +277,6 @@ void ElisaApplication::openRequested(const QList<QUrl> &uris)
     openFiles(uris);
 }
 
-void ElisaApplication::appHelpActivated()
-{
-    QDesktopServices::openUrl(QUrl(QStringLiteral("help:/")));
-}
-
-void ElisaApplication::aboutApplication()
-{
-#if KFXmlGui_FOUND
-    static QPointer<QDialog> dialog;
-    if (!dialog) {
-        dialog = new KAboutApplicationDialog(KAboutData::applicationData(), nullptr);
-        dialog->setAttribute(Qt::WA_DeleteOnClose);
-    }
-    dialog->show();
-#endif
-}
-
 void ElisaApplication::reportBug()
 {
 #if KFXmlGui_FOUND
@@ -313,19 +299,6 @@ void ElisaApplication::configureShortcuts()
 #endif
 }
 
-void ElisaApplication::configureElisa()
-{
-    if (!d->mEngine)
-    {
-        return;
-    }
-
-    QQmlComponent configurationDialogComponent(d->mEngine,
-                                               QUrl(QStringLiteral("qrc:/qml/ElisaConfigurationDialog.qml")));
-
-    configurationDialogComponent.create();
-}
-
 void ElisaApplication::goBack() {}
 
 void ElisaApplication::find() {}
@@ -359,6 +332,7 @@ void ElisaApplication::configChanged()
     Q_EMIT useFavoriteStyleRatingsChanged();
     Q_EMIT embeddedViewChanged();
     Q_EMIT initialViewIndexChanged();
+    Q_EMIT initialFilesViewPathChanged();
 }
 
 DataTypes::EntryDataList ElisaApplication::checkFileListAndMakeAbsolute(const DataTypes::EntryDataList &filesList,
@@ -367,7 +341,7 @@ DataTypes::EntryDataList ElisaApplication::checkFileListAndMakeAbsolute(const Da
     auto filesToOpen = DataTypes::EntryDataList{};
 
     for (const auto &oneFile : filesList) {
-        auto trackUrl = std::get<0>(oneFile)[DataTypes::ResourceRole].toUrl();
+        auto trackUrl = oneFile.musicData[DataTypes::ResourceRole].toUrl();
         if (trackUrl.scheme().isEmpty() || trackUrl.isLocalFile()) {
             auto newFile = QFileInfo(trackUrl.toLocalFile());
             if (trackUrl.scheme().isEmpty()) {
@@ -383,9 +357,9 @@ DataTypes::EntryDataList ElisaApplication::checkFileListAndMakeAbsolute(const Da
             }
 
             if (newFile.exists()) {
-                auto trackData = std::get<0>(oneFile);
+                auto trackData = oneFile.musicData;
                 trackData[DataTypes::ResourceRole] = QUrl::fromLocalFile(newFile.canonicalFilePath());
-                filesToOpen.push_back({trackData, std::get<1>(oneFile), {}});
+                filesToOpen.push_back({trackData, oneFile.title, {}});
             }
         } else {
             filesToOpen.push_back(oneFile);
@@ -400,20 +374,9 @@ void ElisaApplication::initialize()
     initializeModels();
     initializePlayer();
 
-#if defined Q_OS_ANDROID
-    QIcon::setThemeName(QStringLiteral("Breeze"));
-    QIcon::setFallbackSearchPaths({QIcon::themeSearchPaths().first() + QStringLiteral("/icons")});
-    QIcon::setThemeSearchPaths({QIcon::themeSearchPaths().first()});
-#endif
-
     Q_EMIT initializationDone();
 }
 
-void ElisaApplication::setQmlEngine(QQmlEngine *engine)
-{
-    d->mEngine = engine;
-}
-
 void ElisaApplication::initializeModels()
 {
     d->mMusicManager = std::make_unique<MusicListenersManager>();
@@ -554,11 +517,7 @@ bool ElisaApplication::eventFilter(QObject *object, QEvent *event)
 #if KFXmlGui_FOUND
     auto *keyEvent = static_cast<QKeyEvent*>(event);
     auto playPauseAction = d->mCollection.action(QStringLiteral("Play-Pause"));
-#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
     if (keyEvent->key() == Qt::Key_Space && playPauseAction->shortcut()[0].key() == Qt::Key_Space) {
-#else
-    if (keyEvent->key() == Qt::Key_Space && playPauseAction->shortcut()[0] == Qt::Key_Space) {
-#endif
         return true;
     }
 #else
@@ -567,7 +526,7 @@ bool ElisaApplication::eventFilter(QObject *object, QEvent *event)
     return false;
 }
 
-void ElisaApplication::showInFolder(QUrl filePath)
+void ElisaApplication::showInFolder(const QUrl &filePath)
 {
 #if KFKIO_FOUND
     KIO::highlightInFileManager({filePath});
@@ -670,4 +629,10 @@ int ElisaApplication::initialViewIndex() const
     return result;
 }
 
+QString ElisaApplication::initialFilesViewPath() const
+{
+    return Elisa::ElisaConfiguration::initialFilesViewPath();
+}
+
+
 #include "moc_elisaapplication.cpp"
