@@ -15,6 +15,7 @@
 #include "datatypes.h"
 
 #include <QObject>
+#include <QQmlEngine>
 #include <QString>
 
 #include <memory>
@@ -28,9 +29,7 @@ class AudioWrapper;
 class ManageAudioPlayer;
 class ManageMediaPlayerControl;
 class ManageHeaderBar;
-class QQmlEngine;
 class ElisaApplicationPrivate;
-class QSessionManager;
 class KColorSchemeManager;
 class QAbstractItemModel;
 
@@ -39,10 +38,9 @@ class ELISALIB_EXPORT ElisaApplication : public QObject
 
     Q_OBJECT
 
-    Q_PROPERTY(DataTypes::EntryDataList arguments
-               READ arguments
-               WRITE setArguments
-               NOTIFY argumentsChanged)
+    QML_ELEMENT
+
+    QML_SINGLETON
 
     Q_PROPERTY(ElisaUtils::PlayListEntryType embeddedView
                READ embeddedView
@@ -102,23 +100,27 @@ class ELISALIB_EXPORT ElisaApplication : public QObject
                READ initialViewIndex
                NOTIFY initialViewIndexChanged)
 
+    Q_PROPERTY(QString initialFilesViewPath
+               READ initialFilesViewPath
+               NOTIFY initialFilesViewPathChanged)
+
 public:
+
     explicit ElisaApplication(QObject *parent = nullptr);
 
     ~ElisaApplication() override;
 
     Q_INVOKABLE QAction* action(const QString& name);
+    Q_INVOKABLE const QString actionShortcut(QAction *action) const;
 
     Q_INVOKABLE QString iconName(const QIcon& icon);
 
     Q_INVOKABLE void installKeyEventFilter(QObject *object);
 
-    Q_INVOKABLE void showInFolder(QUrl filePath);
+    Q_INVOKABLE void showInFolder(const QUrl &filePath);
 
     bool eventFilter(QObject *object, QEvent *event) override;
 
-    [[nodiscard]] const DataTypes::EntryDataList &arguments() const;
-
     [[nodiscard]] MusicListenersManager *musicManager() const;
 
     [[nodiscard]] MediaPlayList *mediaPlayList() const;
@@ -145,9 +147,9 @@ public:
 
     int initialViewIndex() const;
 
-Q_SIGNALS:
+    [[nodiscard]] QString initialFilesViewPath() const;
 
-    void argumentsChanged();
+Q_SIGNALS:
 
     void musicManagerChanged();
 
@@ -177,31 +179,34 @@ Q_SIGNALS:
 
     void useFavoriteStyleRatingsChanged();
 
-    void commitDataRequest(QSessionManager &manager);
+    void commitDataRequest();
 
     void embeddedViewChanged();
 
     void initialViewIndexChanged();
 
+    void initialFilesViewPathChanged();
+
+    void openAboutAppPage();
+
+    void openAboutKDEPage();
+
+    void configureElisa();
+
 public Q_SLOTS:
 
-    void appHelpActivated();
-
-    void aboutApplication();
-
     void reportBug();
 
     void configureShortcuts();
 
-    void configureElisa();
-
-    void setArguments(const DataTypes::EntryDataList &newArguments);
+    bool openFiles(const QList<QUrl> &files);
+    bool openFiles(const QList<QUrl> &files, const QString &workingDirectory);
 
     void activateActionRequested(const QString &actionName, const QVariant &parameter);
 
     void activateRequested(const QStringList &arguments, const QString &workingDirectory);
 
-    void openRequested(const QList< QUrl > &uris);
+    void openRequested(const QList<QUrl> &uris);
 
     void initialize();
 
@@ -209,7 +214,7 @@ public Q_SLOTS:
 
 public:
 
-    void setQmlEngine(QQmlEngine *engine);
+    KColorSchemeManager *getSchemes();
 
 private Q_SLOTS:
 
