@@ -75,7 +75,6 @@ void ElisaConfigurationDialog::save()
     Elisa::ElisaConfiguration::setShowNowPlayingBackground(mShowNowPlayingBackground);
     Elisa::ElisaConfiguration::setShowProgressOnTaskBar(mShowProgressInTaskBar);
     Elisa::ElisaConfiguration::setShowSystemTrayIcon(mShowSystemTrayIcon);
-    Elisa::ElisaConfiguration::setForceUsageOfFastFileSearch(!mForceUsageOfSlowFileSystemIndexing);
     Elisa::ElisaConfiguration::setAlwaysUseAbsolutePlaylistPaths(mAlwaysUseAbsolutePlaylistPaths);
     Elisa::ElisaConfiguration::setPlayAtStartup(mPlayAtStartup);
     Elisa::ElisaConfiguration::setScanAtStartup(mScanAtStartup);
@@ -108,6 +107,7 @@ void ElisaConfigurationDialog::save()
     }
 
     Elisa::ElisaConfiguration::setInitialView(mInitialViewIndex);
+    Elisa::ElisaConfiguration::setInitialFilesViewPath(mInitialFilesViewPath);
 
     Elisa::ElisaConfiguration::self()->save();
 
@@ -161,18 +161,6 @@ void ElisaConfigurationDialog::setShowSystemTrayIcon(bool showSystemTrayIcon)
     setDirty();
 }
 
-void ElisaConfigurationDialog::setForceUsageOfSlowFileSystemIndexing(bool forceUsageOfSlowFileSystemIndexing)
-{
-    if (mForceUsageOfSlowFileSystemIndexing == forceUsageOfSlowFileSystemIndexing) {
-        return;
-    }
-
-    mForceUsageOfSlowFileSystemIndexing = forceUsageOfSlowFileSystemIndexing;
-    Q_EMIT forceUsageOfSlowFileSystemIndexingChanged();
-
-    setDirty();
-}
-
 void ElisaConfigurationDialog::setAlwaysUseAbsolutePlaylistPaths(bool alwaysUseAbsolutePlaylistPaths)
 {
     if (mAlwaysUseAbsolutePlaylistPaths == alwaysUseAbsolutePlaylistPaths) {
@@ -192,7 +180,7 @@ void ElisaConfigurationDialog::setEmbeddedView(ElisaUtils::PlayListEntryType emb
     }
 
     mEmbeddedView = embeddedView;
-    QTimer::singleShot(0, [this](){ Q_EMIT embeddedViewChanged(); });
+    QTimer::singleShot(0, this, &ElisaConfigurationDialog::embeddedViewChanged);
 
     setDirty();
 }
@@ -204,7 +192,19 @@ void ElisaConfigurationDialog::setInitialViewIndex(int initialViewIndex)
     }
 
     mInitialViewIndex = initialViewIndex;
-    QTimer::singleShot(0, [this](){ Q_EMIT initialViewIndexChanged(); });
+    QTimer::singleShot(0, this, &ElisaConfigurationDialog::initialViewIndexChanged);
+
+    setDirty();
+}
+
+void ElisaConfigurationDialog::setInitialFilesViewPath(const QString &path)
+{
+    if (mInitialFilesViewPath == path) {
+        return;
+    }
+
+    mInitialFilesViewPath = path;
+    Q_EMIT initialFilesViewPathChanged();
 
     setDirty();
 }
@@ -253,7 +253,7 @@ void ElisaConfigurationDialog::setUseFavoriteStyleRatings(bool useFavoriteStyleR
     setDirty();
 }
 
-void ElisaConfigurationDialog::removeMusicLocation(QString location)
+void ElisaConfigurationDialog::removeMusicLocation(const QString &location)
 {
     mRootPath.removeAll(location);
     Q_EMIT rootPathChanged(mRootPath);
@@ -285,9 +285,6 @@ void ElisaConfigurationDialog::configChanged()
     mUseFavoriteStyleRatings = Elisa::ElisaConfiguration::useFavoriteStyleRatings();
     Q_EMIT useFavoriteStyleRatingsChanged();
 
-    mForceUsageOfSlowFileSystemIndexing = !Elisa::ElisaConfiguration::forceUsageOfFastFileSearch();
-    Q_EMIT forceUsageOfSlowFileSystemIndexingChanged();
-
     mAlwaysUseAbsolutePlaylistPaths = Elisa::ElisaConfiguration::alwaysUseAbsolutePlaylistPaths();
     Q_EMIT alwaysUseAbsolutePlaylistPathsChanged();
 
@@ -310,6 +307,9 @@ void ElisaConfigurationDialog::configChanged()
 
     mInitialViewIndex = Elisa::ElisaConfiguration::initialView();
     Q_EMIT initialViewIndexChanged();
+
+    mInitialFilesViewPath = Elisa::ElisaConfiguration::initialFilesViewPath();
+    Q_EMIT initialFilesViewPathChanged();
 }
 
 void ElisaConfigurationDialog::setDirty()
