@@ -13,28 +13,32 @@
 
 #include <QStringList>
 #include <QFileSystemWatcher>
+#include <QQmlEngine>
 
 class ELISALIB_EXPORT ElisaConfigurationDialog : public QObject
 {
 
     Q_OBJECT
 
+    QML_ELEMENT
+
+    QML_SINGLETON
+
     Q_PROPERTY(QStringList rootPath
                READ rootPath
                WRITE setRootPath
                NOTIFY rootPathChanged)
 
-    Q_PROPERTY(bool forceUsageOfSlowFileSystemIndexing
-               READ forceUsageOfSlowFileSystemIndexing
-               WRITE setForceUsageOfSlowFileSystemIndexing
-               NOTIFY forceUsageOfSlowFileSystemIndexingChanged)
+    Q_PROPERTY(bool alwaysUseAbsolutePlaylistPaths
+               READ alwaysUseAbsolutePlaylistPaths
+               WRITE setAlwaysUseAbsolutePlaylistPaths
+               NOTIFY alwaysUseAbsolutePlaylistPathsChanged)
 
-Q_PROPERTY(bool showNowPlayingBackground
+    Q_PROPERTY(bool showNowPlayingBackground
                READ showNowPlayingBackground
                WRITE setShowNowPlayingBackground
                NOTIFY showNowPlayingBackgroundChanged)
 
-
     Q_PROPERTY(bool showProgressInTaskBar
                READ showProgressInTaskBar
                WRITE setShowProgressInTaskBar
@@ -55,6 +59,11 @@ Q_PROPERTY(bool showNowPlayingBackground
                WRITE setInitialViewIndex
                NOTIFY initialViewIndexChanged)
 
+    Q_PROPERTY(QString initialFilesViewPath
+               READ initialFilesViewPath
+               WRITE setInitialFilesViewPath
+               NOTIFY initialFilesViewPathChanged)
+
     Q_PROPERTY(bool isDirty
                READ isDirty
                NOTIFY isDirtyChanged)
@@ -64,13 +73,30 @@ Q_PROPERTY(bool showNowPlayingBackground
                WRITE setPlayAtStartup
                NOTIFY playAtStartupChanged)
 
-        Q_PROPERTY(QString colorScheme
-                   READ colorScheme
-                   WRITE setColorScheme
-                   NOTIFY colorSchemeChanged)
+    Q_PROPERTY(bool scanAtStartup
+               READ scanAtStartup
+               WRITE setScanAtStartup
+               NOTIFY scanAtStartupChanged)
+
+    Q_PROPERTY(QString colorScheme
+               READ colorScheme
+               WRITE setColorScheme
+               NOTIFY colorSchemeChanged)
+
+    Q_PROPERTY(bool useFavoriteStyleRatings
+               READ useFavoriteStyleRatings
+               WRITE setUseFavoriteStyleRatings
+               NOTIFY useFavoriteStyleRatingsChanged)
 
 public:
 
+    static ElisaConfigurationDialog *create(QQmlEngine *engine, QJSEngine *scriptEngine)
+    {
+        Q_UNUSED(engine)
+        Q_UNUSED(scriptEngine)
+        return new ElisaConfigurationDialog;
+    }
+
     explicit ElisaConfigurationDialog(QObject *parent = nullptr);
 
     ~ElisaConfigurationDialog() override;
@@ -98,9 +124,9 @@ public:
         return mShowSystemTrayIcon;
     }
 
-    [[nodiscard]] bool forceUsageOfSlowFileSystemIndexing() const
+    [[nodiscard]] bool alwaysUseAbsolutePlaylistPaths() const
     {
-        return mForceUsageOfSlowFileSystemIndexing;
+        return mAlwaysUseAbsolutePlaylistPaths;
     }
 
     [[nodiscard]] ElisaUtils::PlayListEntryType embeddedView() const
@@ -113,17 +139,32 @@ public:
         return mInitialViewIndex;
     }
 
+    [[nodiscard]] QString initialFilesViewPath() const
+    {
+        return mInitialFilesViewPath;
+    }
+
     [[nodiscard]] bool playAtStartup() const
     {
         return mPlayAtStartup;
     }
 
+    [[nodiscard]] bool scanAtStartup() const
+    {
+        return mScanAtStartup;
+    }
+
     [[nodiscard]] QString colorScheme() const
     {
         return mColorScheme;
     }
 
-    Q_INVOKABLE void removeMusicLocation(QString location);
+    [[nodiscard]] bool useFavoriteStyleRatings() const
+    {
+        return mUseFavoriteStyleRatings;
+    }
+
+    Q_INVOKABLE void removeMusicLocation(const QString &location);
 
 
 Q_SIGNALS:
@@ -138,16 +179,22 @@ Q_SIGNALS:
 
     void showSystemTrayIconChanged();
 
-    void forceUsageOfSlowFileSystemIndexingChanged();
+    void alwaysUseAbsolutePlaylistPathsChanged();
 
     void embeddedViewChanged();
 
     void initialViewIndexChanged();
 
+    void initialFilesViewPathChanged();
+
     void playAtStartupChanged();
 
+    void scanAtStartupChanged();
+
     void colorSchemeChanged();
 
+    void useFavoriteStyleRatingsChanged();
+
 public Q_SLOTS:
 
     void setRootPath(const QStringList &rootPath);
@@ -162,16 +209,22 @@ public Q_SLOTS:
 
     void setShowSystemTrayIcon(bool showSystemTrayIcon);
 
-    void setForceUsageOfSlowFileSystemIndexing(bool forceUsageOfSlowFileSystemIndexing);
+    void setAlwaysUseAbsolutePlaylistPaths(bool alwaysUseAbsolutePlaylistPaths);
 
     void setEmbeddedView(ElisaUtils::PlayListEntryType embeddedView);
 
     void setInitialViewIndex(int initialViewIndex);
 
+    void setInitialFilesViewPath(const QString &path);
+
     void setPlayAtStartup(bool playAtStartup);
 
+    void setScanAtStartup(bool scanAtStartup);
+
     void setColorScheme(const QString &scheme);
 
+    void setUseFavoriteStyleRatings(bool useFavoriteStyleRatings);
+
 private Q_SLOTS:
 
     void configChanged();
@@ -192,15 +245,21 @@ private:
 
     bool mShowSystemTrayIcon = false;
 
-    bool mForceUsageOfSlowFileSystemIndexing = true;
+    bool mAlwaysUseAbsolutePlaylistPaths = false;
 
     bool mPlayAtStartup = false;
 
+    bool mScanAtStartup = true;
+
+    bool mUseFavoriteStyleRatings = false;
+
     QString mColorScheme;
 
     ElisaUtils::PlayListEntryType mEmbeddedView = ElisaUtils::Unknown;
 
     int mInitialViewIndex = 2;
+
+    QString mInitialFilesViewPath;
 };
 
 #endif
