@@ -7,17 +7,27 @@
 #include "config-upnp-qt.h"
 #include "elisa-version.h"
 
+#include "colorschemepreviewimageprovider.h"
 #include "elisaapplication.h"
 #include "elisa_settings.h"
 
 #include "localFileConfiguration/elisaconfigurationdialog.h"
 
+#if KFDBusAddons_FOUND
+#include <KDBusService>
+#endif
+
+#if KFFileMetaData_FOUND
+#include "embeddedcoverageimageprovider.h"
+#endif
+
 //#define QT_QML_DEBUG
 
 #include <KLocalizedString>
 #include <KLocalizedContext>
 
 #include <KAboutData>
+#include <KIconTheme>
 
 #if KFCrash_FOUND
 #include <KCrash>
@@ -37,28 +47,28 @@
 #include <QQmlFileSelector>
 #include <QQuickStyle>
 
-#if QtAndroidExtras_FOUND
-#include <QAndroidService>
-#endif
-
 #include <memory>
 
-#if QtAndroidExtras_FOUND
-#include <QAndroidJniObject>
-#include <QtAndroid>
+#ifdef Q_OS_ANDROID
+#include "android/androidcoverimageprovider.h"
+
+#include <QCoreApplication>
+#include <QJniObject>
+#include <QtCore/private/qandroidextras_p.h>
 #endif
 
-#if defined Q_OS_ANDROID
+#ifdef Q_OS_ANDROID
 int __attribute__((visibility("default"))) main(int argc, char *argv[])
 #else
 int main(int argc, char *argv[])
 #endif
 {
+    KIconTheme::initTheme();
     auto format = QSurfaceFormat::defaultFormat();
     format.setOption(QSurfaceFormat::ResetNotification);
     QSurfaceFormat::setDefaultFormat(format);
 
-#if defined Q_OS_ANDROID
+#ifdef Q_OS_ANDROID
     if(argc > 1 && strcmp(argv[1], "-service") == 0){
         QAndroidService app(argc, argv);
         qInfo() << "Service starting...";
@@ -72,9 +82,8 @@ int main(int argc, char *argv[])
 
     qputenv("QT_LOGGING_RULES", "org.kde.elisa*=true");
 #endif
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
-    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+#ifdef Q_OS_WIN
+    qputenv("QT_LOGGING_RULES", "org.kde.elisa*=true");
 #endif
     qputenv("QT_GSTREAMER_USE_PLAYBIN_VOLUME", "true");
 
@@ -84,19 +93,16 @@ int main(int argc, char *argv[])
     QApplication::setStyle(QStringLiteral("breeze"));
 #endif
 
-    KLocalizedString::setApplicationDomain("elisa");
+    KLocalizedString::setApplicationDomain(QByteArrayLiteral("elisa"));
 
-#if QtAndroidExtras_FOUND
+#ifdef Q_OS_ANDROID
     qInfo() << QCoreApplication::arguments();
 
-    QAndroidJniObject::callStaticMethod<void>("org/kde/elisa/ElisaService",
-                                              "startMyService",
-                                              "(Landroid/content/Context;)V",
-                                              QtAndroid::androidContext().object());
+#if QT_VERSION < QT_VERSION_CHECK(6, 7, 0)
+    QJniObject::callStaticMethod<void>("org/kde/elisa/ElisaService", "startMyService", "(Landroid/content/Context;)V", QNativeInterface::QAndroidApplication::context());
+#else
+    QJniObject::callStaticMethod<void>("org/kde/elisa/ElisaService", "startMyService", QNativeInterface::QAndroidApplication::context());
 #endif
-
-#if KFCrash_FOUND
-    KCrash::initialize();
 #endif
 
     QApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("elisa")));
@@ -118,6 +124,12 @@ int main(int argc, char *argv[])
     aboutData.addCredit(QStringLiteral("Jérôme Guidon"), i18nc("@label", "Support for online radios"), QStringLiteral("guidon@live.fr"));
 
     KAboutData::setApplicationData(aboutData);
+#if KFCrash_FOUND
+    KCrash::initialize();
+#endif
+    qmlRegisterSingletonType("org.kde.elisa.about", 1, 0, "About", [](QQmlEngine *engine, QJSEngine *) -> QJSValue {
+        return engine->toScriptValue(KAboutData::applicationData());
+    });
 
     QCommandLineParser parser;
     aboutData.setupCommandLine(&parser);
@@ -140,20 +152,41 @@ int main(int argc, char *argv[])
     QQmlApplicationEngine engine;
     engine.addImportPath(QStringLiteral("qrc:/imports"));
     QQmlFileSelector selector(&engine);
+    constexpr auto qmlUri = "org.kde.elisa";
+
+#if KFDBusAddons_FOUND
+    const auto *elisaApplication = engine.singletonInstance<ElisaApplication *>(qmlUri, "ElisaApplication");
+
+    KDBusService service(KDBusService::Unique);
+    service.connect(&service, &KDBusService::activateActionRequested, elisaApplication, &ElisaApplication::activateActionRequested);
+    service.connect(&service, &KDBusService::activateRequested, elisaApplication, &ElisaApplication::activateRequested);
+    service.connect(&service, &KDBusService::openRequested, elisaApplication, &ElisaApplication::openRequested);
+#endif
 
     // Allow image:// icon URLs to be loaded as images im QML
     engine.addImageProvider(QStringLiteral("icon"), new KQuickIconProvider);
 
+#if KFFileMetaData_FOUND
+    engine.addImageProvider(QStringLiteral("cover"), new EmbeddedCoverageImageProvider);
+#endif
+
+#ifdef Q_OS_ANDROID
+    engine.addImageProvider(QStringLiteral("android"), new AndroidCoverImageProvider);
+#endif
+
+    engine.addImageProvider(QStringLiteral("colorScheme"), new ColorSchemePreviewImageProvider);
+
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
 
     QList<QUrl> urls;
-    for (const auto &oneArgument : parser.positionalArguments()) {
+    const auto positionalArguments = parser.positionalArguments();
+    for (const auto &oneArgument : positionalArguments) {
         urls.push_back(QUrl::fromUserInput(oneArgument, QDir::currentPath()));
     }
 
     engine.rootContext()->setContextProperty(QStringLiteral("elisaStartupArguments"), QVariant::fromValue(urls));
 
-    engine.load(QUrl(QStringLiteral("qrc:/qml/ElisaMainWindow.qml")));
+    engine.loadFromModule(qmlUri, "ElisaMainWindow");
 
     return app.exec();
 }
