@@ -15,11 +15,6 @@
 #include <QTimer>
 #include <QDateTime>
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-#define PlaybackState State
-#define playbackState state
-#endif
-
 ManageAudioPlayer::ManageAudioPlayer(QObject *parent) : QObject(parent)
 {
 
@@ -417,11 +412,16 @@ void ManageAudioPlayer::setPlayerPosition(qint64 playerPosition)
     QTimer::singleShot(0, this, [this]() {Q_EMIT playControlPositionChanged();});
 }
 
-void ManageAudioPlayer::setCurrentPlayingForRadios(const QString &title, const QString &nowPlaying)
+void ManageAudioPlayer::setCurrentPlayingForRadios(const QString &title, const QString &artistOrStation)
 {
     if (mPlayListModel && mCurrentTrack.isValid()) {
-        Q_EMIT updateData(mCurrentTrack, title, MediaPlayList::TitleRole);
-        Q_EMIT updateData(mCurrentTrack, nowPlaying, MediaPlayList::ArtistRole);
+        // WORKAROUND: Don't update metadata if the title is the last subpath of the stream URL. This is an unresolved issue with libVLC,
+        // see issue 21057 here: https://code.videolan.org/videolan/vlc/-/issues/21057
+        bool suppressMetadataUpdate = !title.isEmpty() && mCurrentTrack.data(mUrlRole).toString().endsWith(title);
+        if (!suppressMetadataUpdate) {
+            Q_EMIT updateData(mCurrentTrack, title, MediaPlayList::TitleRole);
+            Q_EMIT updateData(mCurrentTrack, artistOrStation, MediaPlayList::ArtistRole);
+        }
     }
 }
 
@@ -457,7 +457,7 @@ void ManageAudioPlayer::playListFinished()
     mPlayingState = false;
 }
 
-void ManageAudioPlayer::tracksDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles)
+void ManageAudioPlayer::tracksDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QList<int> &roles)
 {
     if (!mCurrentTrack.isValid()) {
         return;
@@ -551,9 +551,9 @@ void ManageAudioPlayer::triggerStop()
     QTimer::singleShot(0, this, [this]() {Q_EMIT playerStop();});
 }
 
-void ManageAudioPlayer::triggerSkipNextTrack()
+void ManageAudioPlayer::triggerSkipNextTrack(ElisaUtils::SkipReason reason /*= SkipReason::Automatic */)
 {
-    QTimer::singleShot(0, this, [this]() {Q_EMIT skipNextTrack();});
+    QTimer::singleShot(0, this, [this, reason]() {Q_EMIT skipNextTrack(reason);});
 }
 
 void ManageAudioPlayer::restorePreviousState()
