@@ -7,6 +7,7 @@
  */
 
 #include "mediaplaylistproxymodel.h"
+#include "elisautils.h"
 #include "mediaplaylist.h"
 #include "playListLogging.h"
 #include "elisa_settings.h"
@@ -25,11 +26,14 @@
 
 #include <algorithm>
 
+using namespace Qt::Literals::StringLiterals;
+
 QList<QUrl> M3uPlaylistParser::fromPlaylist(const QUrl &fileName, const QByteArray &fileContent) {
     Q_UNUSED(fileName);
     QList<QUrl> result;
 
-    for (const QByteArray &l : fileContent.split('\n')) {
+    const auto lines = fileContent.split('\n');
+    for (const QByteArray &l : lines) {
         const QString &line = QString::fromUtf8(l);
         if (!line.isEmpty() && !line.startsWith(QStringLiteral("#"))) {
             const QUrl &url = line.contains(QStringLiteral("://")) ? QUrl(line) : QUrl::fromLocalFile(line);
@@ -56,7 +60,9 @@ QString M3uPlaylistParser::toPlaylist(const QUrl &fileName, const QList<QString>
 QList<QUrl> PlsPlaylistParser::fromPlaylist(const QUrl &fileName, const QByteArray &fileContent) {
     Q_UNUSED(fileName);
     QList<QUrl> result;
-    for (const QByteArray &l : fileContent.split('\n')) {
+
+    const auto lines = fileContent.split('\n');
+    for (const QByteArray &l : lines) {
         const QString &line = QString::fromUtf8(l);
         if (!line.isEmpty() && line.startsWith(QStringLiteral("File")) && line.contains(QStringLiteral("="))) {
             int indexOfEquals = line.indexOf(QStringLiteral("="));
@@ -164,7 +170,7 @@ public:
 
     MediaPlayListProxyModel::Repeat mRepeatMode = MediaPlayListProxyModel::Repeat::None;
 
-    bool mShufflePlayList = false;
+    MediaPlayListProxyModel::Shuffle mShuffleMode = MediaPlayListProxyModel::Shuffle::NoShuffle;
 
     bool mPartiallyLoaded = false;
 
@@ -230,7 +236,7 @@ QModelIndex MediaPlayListProxyModel::mapToSource(const QModelIndex &proxyIndex)
 
 int MediaPlayListProxyModel::mapRowToSource(const int proxyRow) const
 {
-    if (d->mShufflePlayList) {
+    if (d->mRandomMapping.size() && d->mShuffleMode != MediaPlayListProxyModel::Shuffle::NoShuffle) {
         return d->mRandomMapping.at(proxyRow);
     } else {
         return proxyRow;
@@ -239,7 +245,7 @@ int MediaPlayListProxyModel::mapRowToSource(const int proxyRow) const
 
 int MediaPlayListProxyModel::mapRowFromSource(const int sourceRow) const
 {
-    if (d->mShufflePlayList) {
+    if (d->mShuffleMode != MediaPlayListProxyModel::Shuffle::NoShuffle) {
         return d->mRandomMapping.indexOf(sourceRow);
     } else {
         return sourceRow;
@@ -248,7 +254,7 @@ int MediaPlayListProxyModel::mapRowFromSource(const int sourceRow) const
 
 int MediaPlayListProxyModel::rowCount(const QModelIndex &parent) const
 {
-    if (d->mShufflePlayList) {
+    if (d->mShuffleMode != MediaPlayListProxyModel::Shuffle::NoShuffle) {
         if (parent.isValid()) {
             return 0;
         }
@@ -338,6 +344,7 @@ void MediaPlayListProxyModel::setRepeatMode(Repeat value)
         d->mRepeatMode = value;
         Q_EMIT repeatModeChanged();
         Q_EMIT remainingTracksChanged();
+        Q_EMIT remainingTracksDurationChanged();
         Q_EMIT persistentStateChanged();
         determineAndNotifyPreviousAndNextTracks();
     }
@@ -348,42 +355,17 @@ MediaPlayListProxyModel::Repeat MediaPlayListProxyModel::repeatMode() const
     return d->mRepeatMode;
 }
 
-void MediaPlayListProxyModel::setShufflePlayList(const bool value)
+void MediaPlayListProxyModel::setShuffleMode(const MediaPlayListProxyModel::Shuffle value)
 {
-    if (d->mShufflePlayList != value) {
-        Q_EMIT layoutAboutToBeChanged(QList<QPersistentModelIndex>(), QAbstractItemModel::VerticalSortHint);
+    if (d->mShuffleMode != value) {
         auto playListSize = d->mPlayListModel->rowCount();
 
         if (playListSize != 0) {
-            if (value) {
-                d->mRandomMapping.clear();
-                d->mRandomMapping.reserve(playListSize);
+            Q_EMIT layoutAboutToBeChanged(QList<QPersistentModelIndex>(), QAbstractItemModel::VerticalSortHint);
 
-                QModelIndexList to;
-                to.reserve(playListSize);
-                for (int i = 0; i < playListSize; ++i) {
-                    to.append(index(i,0));
-                    d->mRandomMapping.append(i);
-                }
-
-                QModelIndexList from;
-                from.reserve(playListSize);
-                // Adding the current track first if it is not the only one
-                if (playListSize > 1) {
-                    if (currentTrackRow() != 0) {
-                        std::swap(d->mRandomMapping[0], d->mRandomMapping[currentTrackRow()]);
-                    }
-                    from.append(index(d->mRandomMapping.at(0), 0));
-                }
-                // Fisher-Yates algorithm
-                for (int i = 1;  i < playListSize - 1; ++i) {
-                    const int swapIndex = d->mRandomGenerator.bounded(i, playListSize);
-                    std::swap(d->mRandomMapping[i], d->mRandomMapping[swapIndex]);
-                    from.append(index(d->mRandomMapping.at(i), 0));
-                }
-                from.append(index(d->mRandomMapping.at(playListSize - 1), 0));
-                changePersistentIndexList(from, to);
-            } else {
+            // first, reset playlist to non-random one if it's currently shuffled
+            if (!d->mRandomMapping.isEmpty()) {
+                Q_ASSERT(d->mShuffleMode != MediaPlayListProxyModel::Shuffle::NoShuffle);
                 QModelIndexList from;
                 from.reserve(playListSize);
                 QModelIndexList to;
@@ -393,25 +375,99 @@ void MediaPlayListProxyModel::setShufflePlayList(const bool value)
                     from.append(index(i, 0));
                 }
                 changePersistentIndexList(from, to);
+
                 d->mRandomMapping.clear();
             }
 
+            if (value != MediaPlayListProxyModel::Shuffle::NoShuffle) {
+                QModelIndexList from, to;
+                from.reserve(playListSize);
+                to.reserve(playListSize);
+
+                d->mRandomMapping.clear();
+                d->mRandomMapping.reserve(playListSize);
+
+                for (int i = 0; i < playListSize; ++i) {
+                    d->mRandomMapping.append(i);
+                    to.append(index(i,0));
+                }
+
+                if (value == MediaPlayListProxyModel::Shuffle::Track) { // shuffle tracks
+                    // Adding the current track first if it is not the only one
+                    if (playListSize > 1) {
+                        if (currentTrackRow() != 0) {
+                            std::swap(d->mRandomMapping[0], d->mRandomMapping[currentTrackRow()]);
+                        }
+                        from.append(index(d->mRandomMapping.at(0), 0));
+                    }
+                    // Fisher-Yates algorithm
+                    for (int i = 1;  i < playListSize - 1; ++i) {
+                        const int swapIndex = d->mRandomGenerator.bounded(i, playListSize);
+                        std::swap(d->mRandomMapping[i], d->mRandomMapping[swapIndex]);
+                        from.append(index(d->mRandomMapping.at(i), 0));
+                    }
+                    from.append(index(d->mRandomMapping.at(playListSize - 1), 0));
+                } else if (value == MediaPlayListProxyModel::Shuffle::Album) { // album shuffle
+                    QHash<int, QList<int>> indexPerAlbumId;
+                    int currentAlbumId = data(d->mCurrentTrack,  MediaPlayList::AlbumIdRole).toInt();
+
+                    // Adding the album of the current track first
+                    QList<int> albumIds = {currentAlbumId};
+                    indexPerAlbumId[currentAlbumId] = {};
+
+                    // This is used to generate fictive (negative) albumIds for
+                    // tracks that don't belong to an album; this will allow to
+                    // spread the loose tracks in between full albums rather
+                    // than have them artificially grouped together
+                    int fictiveAlbumId = -1;
+
+                    for (int i = 0; i < playListSize; ++i) {
+                        int albumId = data(index(i, 0), MediaPlayList::AlbumIdRole).toInt();
+                        if (albumId == 0) {
+                            albumId = fictiveAlbumId;
+                            --fictiveAlbumId;
+                        }
+                        if (indexPerAlbumId.contains(albumId)) {
+                            indexPerAlbumId[albumId].append(i);
+                        } else {
+                            albumIds.append(albumId);
+                            QList<int> tracks = { i };
+                            indexPerAlbumId[albumId] = tracks;
+                        }
+                    }
+
+                    std::shuffle(++albumIds.begin(), albumIds.end(), d->mRandomGenerator);
+
+                    QList<int> newRandomMapping;
+                    newRandomMapping.reserve(playListSize);
+                    for (int albumId : albumIds) {
+                        const auto &sourceRows = indexPerAlbumId[albumId];
+                        for (int sourceRow : sourceRows) {
+                            from.append(index(sourceRow, 0));
+                            newRandomMapping.append(sourceRow);
+                        }
+                    }
+                    d->mRandomMapping = newRandomMapping;
+                }
+                changePersistentIndexList(from, to);
+            }
             d->mCurrentPlayListPosition = d->mCurrentTrack.row();
-            d->mShufflePlayList = value;
+            d->mShuffleMode = value;
             Q_EMIT layoutChanged(QList<QPersistentModelIndex>(), QAbstractItemModel::VerticalSortHint);
             determineAndNotifyPreviousAndNextTracks();
         } else {
-            d->mShufflePlayList = value;
+            d->mShuffleMode = value;
         }
-        Q_EMIT shufflePlayListChanged();
+        Q_EMIT shuffleModeChanged();
         Q_EMIT remainingTracksChanged();
+        Q_EMIT remainingTracksDurationChanged();
         Q_EMIT persistentStateChanged();
     }
 }
 
-bool MediaPlayListProxyModel::shufflePlayList() const
+MediaPlayListProxyModel::Shuffle MediaPlayListProxyModel::shuffleMode() const
 {
-    return d->mShufflePlayList;
+    return d->mShuffleMode;
 }
 
 void MediaPlayListProxyModel::sourceRowsAboutToBeInserted(const QModelIndex &parent, int start, int end)
@@ -422,16 +478,17 @@ void MediaPlayListProxyModel::sourceRowsAboutToBeInserted(const QModelIndex &par
      * new items can be added at arbitrarily positions,
      * which requires a split of beginInsertRows
      */
-    if (!d->mShufflePlayList) {
+    if (d->mShuffleMode == MediaPlayListProxyModel::Shuffle::NoShuffle) {
         beginInsertRows(parent, start, end);
     }
 }
 
 void MediaPlayListProxyModel::sourceRowsInserted(const QModelIndex &parent, int start, int end)
 {
-    if (d->mShufflePlayList) {
+    if (d->mShuffleMode == MediaPlayListProxyModel::Shuffle::Track) { // track shuffle
         const auto newItemsCount = end - start + 1;
         d->mRandomMapping.reserve(rowCount() + newItemsCount);
+
         if (rowCount() == 0) {
             beginInsertRows(parent, start, end);
             for (int i = 0; i < newItemsCount; ++i) {
@@ -441,26 +498,167 @@ void MediaPlayListProxyModel::sourceRowsInserted(const QModelIndex &parent, int
             }
             endInsertRows();
         } else {
-            for (int i = 0; i < newItemsCount; ++i) {
-                //QRandomGenerator.bounded(int) is exclusive, thus + 1
-                const auto random = d->mRandomGenerator.bounded(d->mRandomMapping.count()+1);
-                beginInsertRows(parent, random, random);
-                d->mRandomMapping.insert(random, start + i);
+            if (start <= rowCount()) {
+                // First increment old track indices
+                std::transform(d->mRandomMapping.begin(), d->mRandomMapping.end(), d->mRandomMapping.begin(), [=](const int sourceRow) {
+                    return sourceRow < start ? sourceRow : sourceRow + newItemsCount;
+                });
+            }
+
+            const bool enqueueAfterCurrentTrack = mapRowFromSource(start - 1) == d->mCurrentTrack.row();
+
+            if (enqueueAfterCurrentTrack) {
+                // Internally shuffle the new tracks then enqueue them all immediately after the current track
+                QList<int> shuffledSourceRows(newItemsCount);
+                std::iota(shuffledSourceRows.begin(), shuffledSourceRows.end(), start);
+                std::shuffle(shuffledSourceRows.begin(), shuffledSourceRows.end(), d->mRandomGenerator);
+                const int proxyStart = d->mCurrentTrack.row() + 1;
+                beginInsertRows(parent, proxyStart, proxyStart + newItemsCount - 1);
+                for (int sourceRow : shuffledSourceRows) {
+                    d->mRandomMapping.insert(proxyStart, sourceRow);
+                }
                 endInsertRows();
+            } else {
+                const int lowerBound = d->mCurrentTrack.row() + 1;
+                for (int i = 0; i < newItemsCount; ++i) {
+                    //QRandomGenerator.bounded(int) is exclusive, thus + 1
+                    const auto random = d->mRandomGenerator.bounded(lowerBound, rowCount() + 1);
+                    beginInsertRows(parent, random, random);
+                    d->mRandomMapping.insert(random, start + i);
+                    endInsertRows();
+                }
             }
         }
-    } else {
+    } else if (d->mShuffleMode == MediaPlayListProxyModel::Shuffle::Album) { // album shuffle
+        const auto newItemsCount = end - start + 1;
+        d->mRandomMapping.reserve(rowCount() + newItemsCount);
+
+        // This is used to generate fictive (negative) albumIds for
+        // tracks that don't belong to an album; this will allow to
+        // spread the loose tracks in between full albums rather
+        // than have them artificially grouped together
+        int fictiveAlbumId = -1;
+
+        // Sort new rows per album
+        QHash<int, QList<int>> newIndexPerAlbumId;
+        QList<int> newAlbumIds;
+        for (int i = start; i <= end; ++i) {
+            int albumId = d->mPlayListModel->data(d->mPlayListModel->index(i, 0), MediaPlayList::AlbumIdRole).toInt();
+            if (albumId == 0) {
+                albumId = fictiveAlbumId;
+                --fictiveAlbumId;
+            }
+            if (newIndexPerAlbumId.contains(albumId)) {
+                newIndexPerAlbumId[albumId].append(i);
+            } else {
+                newAlbumIds.append(albumId);
+                QList<int> tracks = { i };
+                newIndexPerAlbumId[albumId] = tracks;
+            }
+        }
+
+        if (rowCount() == 0) {
+            std::shuffle(newAlbumIds.begin(), newAlbumIds.end(), d->mRandomGenerator);
+            beginInsertRows(parent, start, end);
+            for (int albumId : newAlbumIds) {
+                d->mRandomMapping += newIndexPerAlbumId.take(albumId);
+            }
+            endInsertRows();
+        } else {
+            if (start <= rowCount()) {
+                // First increment old track indices
+                std::transform(d->mRandomMapping.begin(), d->mRandomMapping.end(), d->mRandomMapping.begin(), [=](const int sourceRow) {
+                    return sourceRow < start ? sourceRow : sourceRow + newItemsCount;
+                });
+            }
+
+            const bool enqueueAfterCurrentTrack = mapRowFromSource(start - 1) == d->mCurrentTrack.row();
+
+            if (enqueueAfterCurrentTrack) {
+                // Internally shuffle the new albums then enqueue them all immediately after the current track
+                std::shuffle(newAlbumIds.begin(), newAlbumIds.end(), d->mRandomGenerator);
+                int proxyRow = d->mCurrentTrack.row() + 1;
+                beginInsertRows(parent, proxyRow, proxyRow + newItemsCount - 1);
+                for (int albumId : newAlbumIds) {
+                    const auto &sourceRows = newIndexPerAlbumId[albumId];
+                    for (int sourceRow : sourceRows) {
+                        d->mRandomMapping.insert(proxyRow++, sourceRow);
+                    }
+                }
+                endInsertRows();
+            } else {
+                // Find all spots in the current shuffled playlist where the album
+                // changes, then insert the new album tracks randomly in these spots
+
+                // Only insert after the current album
+                const int currentAlbumId = data(d->mCurrentTrack, MediaPlayList::AlbumIdRole).toInt();
+                int lowerBound = d->mCurrentTrack.row() + 1;
+                // 0 album ID means there is no associated album, therefore we can insert directly after the current track
+                if (currentAlbumId != 0) {
+                    while (lowerBound < rowCount() && data(index(lowerBound, 0), MediaPlayList::AlbumIdRole).toInt() == currentAlbumId) {
+                        ++lowerBound;
+                    }
+                }
+
+                QList<int> insertIndexes;
+                int previousAlbumId = -1;
+                for (int i = lowerBound; i < rowCount(); ++i) {
+                    int albumId = data(index(i, 0), MediaPlayList::AlbumIdRole).toInt();
+                    if (previousAlbumId != albumId) {
+                        insertIndexes.append(i);
+                        previousAlbumId = albumId;
+                    }
+                }
+                // Also add last index value as possible insertion point
+                insertIndexes.append(rowCount());
+
+                for (int newAlbumId : newAlbumIds) {
+                    const QList<int> &newAlbumTrackIds = newIndexPerAlbumId[newAlbumId];
+
+                    // pick a random spot to insert the new album
+                    const auto random = insertIndexes[d->mRandomGenerator.bounded(insertIndexes.count())];
+
+                    beginInsertRows(parent, random, random + newAlbumTrackIds.count() - 1);
+                    for (int j = 0; j < newAlbumTrackIds.count(); ++j) {
+                        d->mRandomMapping.insert(random + j, newAlbumTrackIds[j]);
+                    }
+                    endInsertRows();
+
+                    // now update insertIndexes because by inserting an album
+                    // all insertion points after the current one will also have
+                    // shifted
+                    for (int j = 0; j < insertIndexes.count(); ++j) {
+                        if (insertIndexes[j] >= random) {
+                            insertIndexes[j] += newAlbumTrackIds.count();
+                        }
+                    }
+                    // finally, also add the current injection point (the
+                    // original was shifted)
+                    insertIndexes.append(random);
+                }
+            }
+        }
+    } else { // no shuffle
         endInsertRows();
     }
-    determineTracks();
+
+    if (d->mCurrentTrack.isValid()) {
+        d->mCurrentPlayListPosition = d->mCurrentTrack.row();
+        determineAndNotifyPreviousAndNextTracks();
+    } else {
+        determineTracks();
+    }
+
     Q_EMIT tracksCountChanged();
+    Q_EMIT totalTracksDurationChanged();
     Q_EMIT remainingTracksChanged();
+    Q_EMIT remainingTracksDurationChanged();
     Q_EMIT persistentStateChanged();
 }
 
 void MediaPlayListProxyModel::sourceRowsAboutToBeRemoved(const QModelIndex &parent, int start, int end)
 {
-    if (d->mShufflePlayList) {
+    if (d->mShuffleMode != MediaPlayListProxyModel::Shuffle::NoShuffle) {
         if (end - start + 1 == rowCount()) {
             beginRemoveRows(parent, start, end);
             d->mRandomMapping.clear();
@@ -492,10 +690,12 @@ void MediaPlayListProxyModel::sourceRowsRemoved(const QModelIndex &parent, int s
     Q_UNUSED(parent);
     Q_UNUSED(start);
     Q_UNUSED(end);
-    if (!d->mShufflePlayList) {
+    if (d->mShuffleMode == MediaPlayListProxyModel::Shuffle::NoShuffle) {
         endRemoveRows();
     }
-    if (!d->mCurrentTrack.isValid()) {
+    if (d->mCurrentTrack.isValid()) {
+        d->mCurrentPlayListPosition = d->mCurrentTrack.row();
+    } else {
         d->mCurrentTrack = index(d->mCurrentPlayListPosition, 0);
         if (d->mCurrentTrack.isValid() && d->mCurrentTrackWasValid) {
             notifyCurrentTrackChanged();
@@ -513,19 +713,21 @@ void MediaPlayListProxyModel::sourceRowsRemoved(const QModelIndex &parent, int s
         determineAndNotifyPreviousAndNextTracks();
     }
     Q_EMIT tracksCountChanged();
+    Q_EMIT totalTracksDurationChanged();
     Q_EMIT remainingTracksChanged();
+    Q_EMIT remainingTracksDurationChanged();
     Q_EMIT persistentStateChanged();
 }
 
 void MediaPlayListProxyModel::sourceRowsAboutToBeMoved(const QModelIndex &parent, int start, int end, const QModelIndex &destParent, int destRow)
 {
-    Q_ASSERT(!d->mShufflePlayList);
+    Q_ASSERT(d->mShuffleMode == MediaPlayListProxyModel::Shuffle::NoShuffle);
     beginMoveRows(parent, start, end, destParent, destRow);
 }
 
 void MediaPlayListProxyModel::sourceRowsMoved(const QModelIndex &parent, int start, int end, const QModelIndex &destParent, int destRow)
 {
-    Q_ASSERT(!d->mShufflePlayList);
+    Q_ASSERT(d->mShuffleMode == MediaPlayListProxyModel::Shuffle::NoShuffle);
     Q_UNUSED(parent);
     Q_UNUSED(start);
     Q_UNUSED(end);
@@ -533,6 +735,7 @@ void MediaPlayListProxyModel::sourceRowsMoved(const QModelIndex &parent, int sta
     Q_UNUSED(destRow);
     endMoveRows();
     Q_EMIT remainingTracksChanged();
+    Q_EMIT remainingTracksDurationChanged();
     Q_EMIT persistentStateChanged();
 }
 
@@ -545,12 +748,14 @@ void MediaPlayListProxyModel::sourceModelReset()
     endResetModel();
 }
 
-void MediaPlayListProxyModel::sourceDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles)
+void MediaPlayListProxyModel::sourceDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QList<int> &roles)
 {
     auto startSourceRow = topLeft.row();
     auto endSourceRow = bottomRight.row();
     for (int i = startSourceRow; i <= endSourceRow; i++) {
         Q_EMIT dataChanged(index(mapRowFromSource(i), 0), index(mapRowFromSource(i), 0), roles);
+        Q_EMIT remainingTracksDurationChanged();
+        Q_EMIT totalTracksDurationChanged();
         if (i == d->mCurrentTrack.row()) {
             Q_EMIT currentTrackDataChanged();
         } else if (i == d->mNextTrack.row()) {
@@ -577,6 +782,24 @@ void MediaPlayListProxyModel::sourceHeaderDataChanged(Qt::Orientation orientatio
     Q_EMIT headerDataChanged(orientation, first, last);
 }
 
+int MediaPlayListProxyModel::totalTracksDuration() const
+{
+    int time = 0;
+    for (int i = 0; i < rowCount(); i++) {
+        time += d->mPlayListModel->data(index(i, 0), MediaPlayList::DurationRole).toTime().msecsSinceStartOfDay();
+    }
+    return time;
+}
+
+int MediaPlayListProxyModel::remainingTracksDuration() const
+{
+    int time = 0;
+    for (int i = d->mCurrentTrack.row(); i < rowCount(); i++) {
+        time += d->mPlayListModel->data(index(i, 0), MediaPlayList::DurationRole).toTime().msecsSinceStartOfDay();
+    }
+    return time;
+}
+
 int MediaPlayListProxyModel::remainingTracks() const
 {
     if (!d->mCurrentTrack.isValid() || (d->mRepeatMode == Repeat::One) || (d->mRepeatMode == Repeat::Playlist)) {
@@ -586,6 +809,17 @@ int MediaPlayListProxyModel::remainingTracks() const
     }
 }
 
+int MediaPlayListProxyModel::radioCount() const
+{
+    int count = 0;
+    for (int i = 0; i < rowCount(); i++) {
+        if (d->mPlayListModel->data(index(i, 0), MediaPlayList::ElementTypeRole) == ElisaUtils::Radio) {
+            count++;
+        }
+    }
+    return count;
+}
+
 int MediaPlayListProxyModel::tracksCount() const
 {
     return rowCount();
@@ -625,7 +859,9 @@ void MediaPlayListProxyModel::enqueue(const DataTypes::EntryDataList &newEntries
         }
     }
 
-    d->mPlayListModel->enqueueMultipleEntries(newEntries);
+    const int enqueueIndex = enqueueMode == ElisaUtils::AfterCurrentTrack ? mapRowToSource(d->mCurrentTrack.row()) + 1 : -1;
+
+    d->mPlayListModel->enqueueMultipleEntries(newEntries, enqueueIndex);
 }
 
 void MediaPlayListProxyModel::trackInError(const QUrl &sourceInError, QMediaPlayer::Error playerError)
@@ -633,17 +869,19 @@ void MediaPlayListProxyModel::trackInError(const QUrl &sourceInError, QMediaPlay
     d->mPlayListModel->trackInError(sourceInError, playerError);
 }
 
-void MediaPlayListProxyModel::skipNextTrack()
+void MediaPlayListProxyModel::skipNextTrack(ElisaUtils::SkipReason reason /*= ElisaUtils::SkipReason::Automatic*/)
 {
     if (!d->mCurrentTrack.isValid()) {
         return;
     }
 
-    if (d->mCurrentTrack.row() >= rowCount() - 1) {
+    if (d->mRepeatMode == Repeat::One && reason == ElisaUtils::SkipReason::Automatic) {
+        // case 1: repeat current track
+        d->mCurrentTrack = index(d->mCurrentTrack.row(), 0);
+    } else if (d->mCurrentTrack.row() >= rowCount() - 1) {
+        // case 2: end of playlist, check if we loop back
         switch (d->mRepeatMode) {
         case Repeat::One:
-            d->mCurrentTrack = index(d->mCurrentTrack.row(), 0);
-            break;
         case Repeat::Playlist:
             d->mCurrentTrack = index(0, 0);
             break;
@@ -653,11 +891,8 @@ void MediaPlayListProxyModel::skipNextTrack()
             break;
         }
     } else {
-        if (d->mRepeatMode == Repeat::One) {
-            d->mCurrentTrack = index(d->mCurrentTrack.row(), 0);
-        } else {
-            d->mCurrentTrack = index(d->mCurrentTrack.row() + 1, 0);
-        }
+        // default case: start playing next track
+        d->mCurrentTrack = index(d->mCurrentTrack.row() + 1, 0);
     }
 
     notifyCurrentTrackChanged();
@@ -681,11 +916,7 @@ void MediaPlayListProxyModel::skipPreviousTrack(qint64 position)
             return;
         }
     } else {
-        if (d->mRepeatMode == Repeat::One) {
-            d->mCurrentTrack = index(d->mCurrentTrack.row(), 0);
-        } else {
-            d->mCurrentTrack = index(d->mCurrentTrack.row() - 1, 0);
-        }
+        d->mCurrentTrack = index(d->mCurrentTrack.row() - 1, 0);
     }
 
     notifyCurrentTrackChanged();
@@ -717,16 +948,23 @@ void MediaPlayListProxyModel::removeRow(int row)
 
 void MediaPlayListProxyModel::moveRow(int from, int to)
 {
-    if (d->mShufflePlayList) {
+    const bool currentTrackIndexChanged = from < to ? (from <= d->mCurrentTrack.row() && d->mCurrentTrack.row() <= to)
+                                                    : (to <= d->mCurrentTrack.row() && d->mCurrentTrack.row() <= from);
+
+    if (d->mShuffleMode != MediaPlayListProxyModel::Shuffle::NoShuffle) {
         beginMoveRows({}, from, from, {}, from < to ? to + 1 : to);
         d->mRandomMapping.move(from, to);
         endMoveRows();
     } else {
         d->mPlayListModel->moveRows({}, from, 1, {}, from < to ? to + 1 : to);
     }
+
+    if (currentTrackIndexChanged) {
+        notifyCurrentTrackRowChanged();
+    }
 }
 
-void MediaPlayListProxyModel::notifyCurrentTrackChanged()
+void MediaPlayListProxyModel::notifyCurrentTrackRowChanged()
 {
     if (d->mCurrentTrack.isValid()) {
         d->mCurrentPlayListPosition = d->mCurrentTrack.row();
@@ -734,9 +972,15 @@ void MediaPlayListProxyModel::notifyCurrentTrackChanged()
         d->mCurrentPlayListPosition = -1;
     }
     determineAndNotifyPreviousAndNextTracks();
-    Q_EMIT currentTrackChanged(d->mCurrentTrack);
     Q_EMIT currentTrackRowChanged();
     Q_EMIT remainingTracksChanged();
+    Q_EMIT remainingTracksDurationChanged();
+}
+
+void MediaPlayListProxyModel::notifyCurrentTrackChanged()
+{
+    notifyCurrentTrackRowChanged();
+    Q_EMIT currentTrackChanged(d->mCurrentTrack);
 }
 
 void MediaPlayListProxyModel::determineAndNotifyPreviousAndNextTracks()
@@ -897,7 +1141,7 @@ void MediaPlayListProxyModel::loadPlayList(const QUrl &fileName,
     resetPartiallyLoaded();
 
     QFile inputFile(fileName.toLocalFile());
-    bool open = inputFile.open(QFile::ReadOnly);
+    bool open = inputFile.open(QFile::ReadOnly | QIODevice::Text);
     if (!open) {
         Q_EMIT playListLoadFailed();
     }
@@ -927,8 +1171,9 @@ QVariantMap MediaPlayListProxyModel::persistentState() const
     QVariantMap currentState;
 
     currentState[QStringLiteral("playList")] = d->mPlayListModel->getEntriesForRestore();
+    currentState[QStringLiteral("shuffleMode")] = d->mShuffleMode;
+    currentState[QStringLiteral("randomMapping")] = getRandomMappingForRestore();
     currentState[QStringLiteral("currentTrack")] = d->mCurrentPlayListPosition;
-    currentState[QStringLiteral("shufflePlayList")] = d->mShufflePlayList;
     currentState[QStringLiteral("repeatMode")] = d->mRepeatMode;
 
     return currentState;
@@ -943,6 +1188,12 @@ void MediaPlayListProxyModel::setPersistentState(const QVariantMap &persistentSt
         d->mPlayListModel->enqueueRestoredEntries(playListIt.value().toList());
     }
 
+    auto shuffleModeStoredValue = persistentStateValue.find(QStringLiteral("shuffleMode"));
+    auto shuffleRandomMappingIt = persistentStateValue.find(QStringLiteral("randomMapping"));
+    if (shuffleModeStoredValue != persistentStateValue.end() && shuffleRandomMappingIt != persistentStateValue.end()) {
+        restoreShuffleMode(shuffleModeStoredValue->value<Shuffle>(), shuffleRandomMappingIt.value().toList());
+    }
+
     auto playerCurrentTrack = persistentStateValue.find(QStringLiteral("currentTrack"));
     if (playerCurrentTrack != persistentStateValue.end()) {
         auto newIndex = index(playerCurrentTrack->toInt(), 0);
@@ -952,11 +1203,6 @@ void MediaPlayListProxyModel::setPersistentState(const QVariantMap &persistentSt
         }
     }
 
-    auto shufflePlayListStoredValue = persistentStateValue.find(QStringLiteral("shufflePlayList"));
-    if (shufflePlayListStoredValue != persistentStateValue.end()) {
-        setShufflePlayList(shufflePlayListStoredValue->toBool());
-    }
-
     auto repeatPlayStoredValue = persistentStateValue.find(QStringLiteral("repeatPlay"));
     if (repeatPlayStoredValue != persistentStateValue.end() && repeatPlayStoredValue->value<bool>()) {
         setRepeatMode(Repeat::Playlist);
@@ -970,6 +1216,50 @@ void MediaPlayListProxyModel::setPersistentState(const QVariantMap &persistentSt
     Q_EMIT persistentStateChanged();
 }
 
+QVariantList MediaPlayListProxyModel::getRandomMappingForRestore() const
+{
+    QVariantList randomMapping;
+
+    if (d->mShuffleMode != MediaPlayListProxyModel::Shuffle::NoShuffle) {
+        randomMapping.reserve(d->mRandomMapping.count());
+        for (int i = 0; i < d->mRandomMapping.count(); ++i) {
+            randomMapping.append(QVariant(d->mRandomMapping[i]));
+        }
+    }
+
+    return randomMapping;
+}
+
+void MediaPlayListProxyModel::restoreShuffleMode(MediaPlayListProxyModel::Shuffle mode, QVariantList mapping)
+{
+    auto playListSize = rowCount();
+
+    if (mode != MediaPlayListProxyModel::Shuffle::NoShuffle && mapping.count() == playListSize && d->mRandomMapping.isEmpty()) {
+        Q_EMIT layoutAboutToBeChanged(QList<QPersistentModelIndex>(), QAbstractItemModel::VerticalSortHint);
+
+        QModelIndexList from, to;
+        from.reserve(playListSize);
+        to.reserve(playListSize);
+        d->mRandomMapping.clear();
+        d->mRandomMapping.reserve(playListSize);
+
+        for (int i = 0; i < playListSize; ++i) {
+            d->mRandomMapping.append(mapping[i].toInt());
+            from.append(index(mapping[i].toInt(), 0));
+            to.append(index(i, 0));
+        }
+        changePersistentIndexList(from, to);
+
+        d->mShuffleMode = mode;
+
+        Q_EMIT layoutChanged(QList<QPersistentModelIndex>(), QAbstractItemModel::VerticalSortHint);
+
+        Q_EMIT shuffleModeChanged();
+        Q_EMIT remainingTracksChanged();
+        Q_EMIT remainingTracksDurationChanged();
+    }
+}
+
 bool MediaPlayListProxyModel::partiallyLoaded() const
 {
     return d->mPartiallyLoaded;
@@ -986,7 +1276,7 @@ bool MediaPlayListProxyModel::canOpenLoadedPlaylist() const
 void MediaPlayListProxyModel::openLoadedPlayList()
 {
 #if KFKIO_FOUND
-    auto job = new KIO::OpenUrlJob(d->mLoadedPlayListUrl, QLatin1String("text/plain"));
+    auto job = new KIO::OpenUrlJob(d->mLoadedPlayListUrl, u"text/plain"_s);
     job->start();
 #endif
 }
@@ -1044,7 +1334,7 @@ void MediaPlayListProxyModel::loadLocalPlayList(DataTypes::EntryDataList &newTra
         d->mPartiallyLoaded = true;
     }
 
-    for (const QUrl &oneUrl : qAsConst(listOfUrls)) {
+    for (const QUrl &oneUrl : std::as_const(listOfUrls)) {
         if (oneUrl.isLocalFile()) {
             QFileInfo fileInfo(oneUrl.toLocalFile());
             loadLocalFile(newTracks, processedFiles, fileInfo);
@@ -1057,7 +1347,7 @@ void MediaPlayListProxyModel::loadLocalPlayList(DataTypes::EntryDataList &newTra
 void MediaPlayListProxyModel::loadLocalDirectory(DataTypes::EntryDataList &newTracks, QSet<QString> &processedFiles, const QUrl &dirName)
 {
     QDir dirInfo(dirName.toLocalFile());
-    auto fileInfoList = dirInfo.entryInfoList(QDir::NoDotAndDotDot | QDir::Readable | QDir::Files | QDir::Dirs, QDir::Name);
+    const auto fileInfoList = dirInfo.entryInfoList(QDir::NoDotAndDotDot | QDir::Readable | QDir::Files | QDir::Dirs, QDir::Name);
 
     for (const auto &fileInfo : fileInfoList) {
         loadLocalFile(newTracks, processedFiles, fileInfo);
