@@ -118,7 +118,7 @@ public Q_SLOTS:
 
     void updateSingleFileMetaData(const QUrl &url, DataTypes::ColumnsRoles role, const QVariant &data);
 
-    void trackHasBeenModified(ListTrackDataType trackDataType, const QHash<QString, QUrl> &covers);
+    void trackHasBeenModified(ModelDataLoader::ListTrackDataType trackDataType, const QHash<QString, QUrl> &covers);
 
 private Q_SLOTS:
 
