@@ -117,10 +117,13 @@ QVariant ViewsModel::data(const QModelIndex &index, int role) const
             case ElisaUtils::FileName:
             case ElisaUtils::Lyricist:
             case ElisaUtils::Container:
+            case ElisaUtils::PlayList:
                 break;
             }
         }
-
+        break;
+    case DataTypeRole:
+        result = d->mViewsData->dataType(index.row());
         break;
     }
 
